@@ -23,7 +23,7 @@
 #include "swf.h"
 
 #ifdef RCSID
-static char *rcsid = "$Id: displaylist.cc,v 1.4 1999/09/03 15:17:40 ode Exp $";
+static char *rcsid = "$Id: displaylist.cc,v 1.2 2004/12/22 18:36:41 tgc Exp $";
 #endif
 
 #define PRINT 0
@@ -111,7 +111,7 @@ void transformBoundingBox(Rect *bb, Matrix *matrix, Rect *boundary, int reset)
         bb->reset();
     }
     
-    if (boundary->xmin != LONG_MAX) {
+    if (boundary->xmin != LONG_MAX && matrix != NULL) {
         bbox(bb, matrix, boundary->xmin, boundary->ymin);
         bbox(bb, matrix, boundary->xmax, boundary->ymin);
         bbox(bb, matrix, boundary->xmin, boundary->ymax);
