@@ -8,7 +8,7 @@
 //  
 
 #ifdef RCSID
-static char *rcsid = "$Id: script.cc,v 1.8 1999/09/10 06:01:21 ode Exp $";
+static char *rcsid = "$Id: script.cc,v 1.6 2004/12/22 18:32:44 tgc Exp $";
 #endif
 
 #define printf(fmt,args...)
@@ -59,6 +59,11 @@ CInputScript::CInputScript(int level)
     m_fileSize = 0;
     m_fileVersion = 0;
 
+    // Initialize file decompression.
+    m_zBuffer = NULL;
+    m_zInitialized = false;
+    m_lastSize = 0;
+
     // Initialize the bit position and buffer.
     m_bitPos = 0;
     m_bitBuf = 0;
@@ -73,12 +78,20 @@ CInputScript::CInputScript(int level)
     m_dumpGuts = false;
 
     needHeader = 1;
+    needFileID = TRUE;
     program = 0;
 
     outOfMemory = 0;
 
     next = NULL;
 
+    // Streaming sound
+    streamID=3000;
+    streamNew=0;
+
+    m_bCompressed = FALSE;
+    m_bExe = FALSE;
+
     return;
 }
 
@@ -93,6 +106,20 @@ CInputScript::~CInputScript(void)
         m_fileBuf = NULL;
         m_fileSize = 0;
     }
+
+    // Shut down the decompresser if it is running.
+    if (m_zInitialized)
+    {
+	inflateEnd(&m_zstream);
+	m_zInitialized = false;
+    }
+
+    // Free the decompression buffer if it is there.
+    if (m_zBuffer)
+    {
+	delete m_zBuffer;
+	 m_zBuffer = NULL;
+    }
 }
 
 
@@ -1469,7 +1496,7 @@ void CInputScript::ParseDefineSound()
 {
     Sound		*sound;
     U32 tagid = (U32) GetWord();
-    long		 nbSamples;
+    long		 nbSamples,skipSize;
     long		 flags;
     char		*buffer;
 
@@ -1478,24 +1505,52 @@ void CInputScript::ParseDefineSound()
     flags = GetByte();
     sound->setSoundFlags(flags);
 
-    nbSamples = GetDWord();
-    buffer = sound->setNbSamples(nbSamples);
-    if (buffer == NULL) {
-    	outOfMemory = 1;
-	delete sound;
-	return;
-    }
-
-    if (flags & soundIsADPCMCompressed) {
-	Adpcm		*adpcm;
-
-	adpcm = new Adpcm( &m_fileBuf[m_filePos] , flags & soundIsStereo );
-
-	adpcm->Decompress((short *)buffer, nbSamples);
-
-	delete adpcm;
-    } else {
-	memcpy(buffer, &m_fileBuf[m_filePos], m_tagLen-5);
+    switch (sound->getFormat()) {
+    	// Raw
+    	case 0:
+	case 3:
+		nbSamples = GetDWord();
+		buffer = sound->setNbSamples(nbSamples);
+		if (buffer == NULL) {
+			outOfMemory = 1;
+			delete sound;
+			return;
+		}
+		memcpy(buffer, &m_fileBuf[m_filePos], m_tagLen-5);
+		break;
+	// ADPCM
+	case 1:
+		nbSamples = GetDWord();
+		buffer = sound->setNbSamples(nbSamples);
+		if (buffer == NULL) {
+			outOfMemory = 1;
+			delete sound;
+			return;
+		}
+		Adpcm		*adpcm;
+		adpcm = new Adpcm( &m_fileBuf[m_filePos] , flags & soundIsStereo );
+		adpcm->Decompress((short *)buffer, nbSamples);
+		delete adpcm;
+		break;
+	// MP3
+	case 2:
+		nbSamples = GetDWord();
+		buffer = sound->setBuffer(m_tagLen-4);
+		if (buffer == NULL) {
+			outOfMemory = 1;
+			delete sound;
+			return;
+		}
+		skipSize = GetWord();
+		fprintf(stderr, "New block\n");
+		sound->setSoundSize(m_tagLen-4);
+		memcpy(buffer, &m_fileBuf[m_filePos], m_tagLen-4);
+		break;
+	
+	// Nellymoser ASAO
+	case 6:
+		fprintf(stderr, "Unsupported sound compression: Nellymoser\n");
+		break;
     }
 
     addCharacter(sound);
@@ -1578,37 +1633,97 @@ void CInputScript::ParseDefineButtonSound()
 
 void CInputScript::ParseSoundStreamHead()
 {
-    int mixFormat = GetByte();
+    int tmp = GetByte();
+    tmp = GetByte();
+    if (tmp != 0) {
+    	streamFlags = tmp;
+	streamNew = 1;
+	streamID++;
+	fprintf(stderr, "new stream 1, id: %d\n", streamID);
+    }
 
-    // The stream settings
-    int format = GetByte();
-    int nSamples = GetWord();
-
-    mixFormat = mixFormat;
-    format = format;
-    nSamples = nSamples;
-
-    printf("tagSoundStreamHead \tmixFrmt %-3u frmt  %-3u nSamples %-5u\n", mixFormat, format, nSamples);
 }
 
 void CInputScript::ParseSoundStreamHead2()
 {
-    int mixFormat = GetByte();
+    int tmp = GetByte();
+    tmp = GetByte();
+    if (tmp != 0) {
+    	streamFlags = tmp;
+	streamNew = 1;
+	streamID++;
+	fprintf(stderr, "new stream 2, id: %d\n", streamID);
+    }
 
-    // The stream settings
-    int format = GetByte();
-    int nSamples = GetWord();
-
-    mixFormat = mixFormat;
-    format = format;
-    nSamples = nSamples;
-
-    //printf("tagSoundStreamHead2 \tmixFormat %-3u format %-3u nSamples %-5u\n", mixFormat, format, nSamples);
 }
 
 void CInputScript::ParseSoundStreamBlock()
 {
-    printf("tagSoundStreamBlock\n");
+    Sound		*sound;
+    long		 skipSize, oldsize, this_frame, next_frame;
+    char		*buffer;
+    
+    if (m_tagLen < 5) return;
+    
+    skipSize = GetWord();
+    skipSize = GetWord();
+    
+
+    if (streamNew) {
+    	fprintf(stderr, "samples: %d\n", skipSize);
+ 	sound = new Sound(streamID);
+
+	sound->setSoundFlags(streamFlags);
+
+	buffer = sound->setBuffer(m_tagLen-4);
+	if (buffer == NULL) {
+	  	outOfMemory = 1;
+		delete sound;
+		return;
+	}
+	sound->setSoundSize(m_tagLen-4);
+    	memcpy(buffer, &m_fileBuf[m_filePos], m_tagLen-4);
+	addCharacter(sound);
+
+	Control *ctrl;
+	streamNew=0;
+
+	ctrl = new Control;
+    	if (ctrl == NULL) {
+		outOfMemory = 1;
+    		return;
+    	}
+    	ctrl->character = getCharacter(streamID);
+    	ctrl->type = ctrlStartSound;
+
+    	program->addControlInCurrentFrame(ctrl);
+    } else {
+    	sound = (Sound*) getCharacter(streamID);
+	oldsize = sound->getSoundSize();
+	buffer = sound->resetBuffer(m_tagLen-4);
+    	memcpy(buffer+oldsize, &m_fileBuf[m_filePos], m_tagLen-4);
+
+	// If playback already has started, we must update 
+	// the SoundList as well...
+	if (sound->getPlaybackStarted()) {
+		SoundList* sl = sound->getSound();
+		this_frame =  (char*) sl->mp3Stream.this_frame - sl->originalMp3;
+		next_frame =  (char*) sl->mp3Stream.next_frame - sl->originalMp3;
+		
+		sl->currentMp3 = sound->getSamples();
+		sl->originalMp3 = sl->currentMp3;
+
+		sl->remainingMp3 += m_tagLen-4;
+
+		mad_stream_buffer (&sl->mp3Stream, (unsigned char*)sl->currentMp3, sl->remainingMp3);
+
+		sl->mp3Stream.this_frame = sl->mp3Stream.this_frame + this_frame;
+		sl->mp3Stream.next_frame = sl->mp3Stream.next_frame + next_frame;
+		
+	}
+    }
+
+
 }
 
 void CInputScript::ParseDefineButtonCxform()
@@ -1920,12 +2035,9 @@ CInputScript::ParseTags(int *status)
 int
 CInputScript::ParseData(FlashMovie *movie, char * data, long size)
 {
+    static long lLastSize = 0;
     int status = FLASH_PARSE_ERROR;
-
-    int z_status;
-    uLongf z_outsize;
-
-    char *z_data, *tmp_data;
+    int zstatus;
 
     m_fileBuf = (unsigned char *)data;
     m_actualSize = size;
@@ -1939,24 +2051,49 @@ CInputScript::ParseData(FlashMovie *movie, char * data, long size)
 
 	    U8 fileHdr[8];
 
-	    memcpy(fileHdr,data,8);
 
-	    // Verify the header and get the file size.
-	    if (((fileHdr[0] != 'F') && (fileHdr[0] != 'C')) 
-	    	|| fileHdr[1] != 'W' || fileHdr[2] != 'S' )
+	    // recognize TRUE pure SWF files
+	    if( data[1] == 'W' && data[2] == 'S' && 0 == lLastSize )
 	    {
-		//fprintf(stderr, "Not a Flash File.\n");
-		return FLASH_PARSE_ERROR;	// Error
-	    }
-	    else
-	    {
-		// Get the file version.
-		m_fileVersion = (U16) fileHdr[3];
+		if(data[0] == 'C' || data[0] == 'F')
+		{
+		    needFileID = FALSE;
+		    memcpy(fileHdr,data,8);
+		    m_filePos = 8;
+		}
 	    }
 
-	    // Get the file size.
+	    if( needFileID )
+	    {
+		char * pos = data + lLastSize;
+		char * end = data + size - 21;
+		// try to see if we have an embeded SWF file (exe)
+		for(  ; pos < end && (pos[0]!='F' || pos[1]!='W' || pos[2]!='S'); pos++ );
+
+		if( pos < end )
+		{
+		    // if we are here then we've found a flash header
+		    memcpy(fileHdr, pos, 8);
+		    m_filePos = 8+(pos-data);
+		    m_bExe = TRUE; // Add file format field info]
+		    needFileID = FALSE;
+		    lLastSize = pos-data; // remember beginning of SWF data in the file
+		}
+		else
+		{
+		    lLastSize = size; // remember where we were
+		    return FLASH_PARSE_NEED_DATA;
+		}
+	    }
+
+	    m_bCompressed = (fileHdr[0] == 'C')?TRUE:FALSE;
+
+	    // Get the file version.
+	    m_fileVersion = (U16) fileHdr[3];
+    
+    	    // Get the file size.
 	    m_fileSize = (U32) fileHdr[4]
-	    	      | ((U32) fileHdr[5] << 8)
+	              | ((U32) fileHdr[5] << 8)
 		      | ((U32) fileHdr[6] << 16)
 		      | ((U32) fileHdr[7] << 24);
 
@@ -1964,68 +2101,122 @@ CInputScript::ParseData(FlashMovie *movie, char * data, long size)
 	    // Verify the minimum length of a Flash file.
 	    if (m_fileSize < 21)
 	    {
-		//printf("ERROR: File size is too short\n");
 		return FLASH_PARSE_ERROR;	// Error
 	    }
 
-	    // Decompress the file if it is compressed
-	    if (fileHdr[0] == 'C') {
-		// Copy the compressed data to a tmp-container
-		tmp_data=(char*)malloc(size);
-		memcpy(tmp_data, data, size);
+	    if( m_bCompressed )
+	    {
+		fprintf(stdout, "decoding...\n");
+		// The file is compressed. Create a buffer to hold the
+		// uncompressed data.
 
-		// Make the output-buffer ready
-		z_data=(char*)malloc(m_fileSize-8);
-		z_outsize=m_fileSize-8;
-
-		// Uncompress using zlib
-		z_status = uncompress((Bytef*)z_data, &z_outsize,(Bytef*) &tmp_data[8], size-8);
-
-		// Check for error, if any it must be because 
-		// the file/data hasen't been loaded fully - i hope
-		if (z_status < 0) {
-			free(tmp_data);
-			free(z_data);
-			return FLASH_PARSE_NEED_DATA;
+		m_zBuffer = new U8 [m_fileSize];
+		if(m_zBuffer == NULL)
+		{
+    		    return FLASH_PARSE_OOM;
 		}
-		// Clean up and move the header of the file
-		// plus the decompressed data to a new container
-		free(tmp_data);
-		data=(char*)realloc(data, m_fileSize);
-		memcpy(data, fileHdr, 8);
-		memcpy(&data[8],z_data, (size_t)z_outsize);
-		free(z_data);
 
-		// Update some 'global' variables
-		m_fileBuf = (unsigned char *)data;
-		m_actualSize = m_fileSize;
+		// Initialize the ZLib decompressor. We skip the first eight
+		// bytes, since we've already processed them.
 
+		m_zstream.next_out = (Bytef *)&m_zBuffer[8];
+		m_zstream.avail_out = (uInt)m_fileSize;
+		m_zstream.next_in = (Bytef *)&data[8];
+		m_zstream.avail_in = 0;
+		m_zstream.zalloc = Z_NULL;
+		m_zstream.zfree = Z_NULL;
+		m_zstream.opaque = Z_NULL;
+
+		zstatus = inflateInit(&m_zstream);
+		if(zstatus != Z_OK)
+		{
+		    return FLASH_PARSE_OOM;
+		}
+		m_zInitialized = true;
+
+		// We'll be decoding from our own decompressed data buffer.
+
+		m_fileBuf = m_zBuffer;
+		m_lastSize = 8;
 	    }
-	    // Set the file position past the header and size information.
-	    m_filePos = 8;
+	    else
+	    {
+		// The file is uncompressed, so we'll be decoding it from the
+		// caller's buffer.
 
-	    // The header is no longer needed size the info has been extracted,
-	    // and at this point there is no danger of a "return FLASH_PARSE_NEED_DATA"
-	    needHeader = 0;	// Yes
+		m_fileBuf = (U8 *)data;
+	    }
+	}
+
+	// If we're reading a compressed file, we need to decompress as much of
+	// it as we have available into the decompressed data buffer.
+	if( m_zBuffer )
+	{
+	    // Figure out how much new compressed data is available.
+
+	    m_zstream.avail_in = (uInt)(size - m_lastSize);
+	    m_lastSize = size;
+
+	    // Decompress it.
+
+	    zstatus = inflate(&m_zstream, Z_SYNC_FLUSH);
+	    if(   (zstatus != Z_OK) && (zstatus != Z_STREAM_END) &&
+					(zstatus != Z_BUF_ERROR))
+	    {
+		// There was an error in decompression.
+
+		return FLASH_PARSE_ERROR;
+	    }
+
+	    // Figure out how much decompressed data is available.
+	    m_actualSize = m_zstream.total_out + 8;
+	}
+	else
+	{
+	    // The file is uncompressed, so the amount of it available for
+	    // processing is just what the caller thinks it is.
+	    // except if we are embeded in another file
+	    m_actualSize = size;// - lLastSize;
+	}
+
+	if( needHeader )
+	{
+	    // Do we have sufficient data to read the header ?
+	    if( m_actualSize < 21 )
+	    {
+		return FLASH_PARSE_NEED_DATA; // No, need more data
+	    }
 
 	    // Get the frame information.
 	    GetRect(&frameRect);
-
 	    frameRate = GetWord() >> 8;
-
 	    frameCount = GetWord();
 
 	    program = new Program(movie, frameCount);
 
-	    if (program == NULL || program->totalFrames == 0) {
-	    	return FLASH_PARSE_ERROR;
+	    if (program == NULL || program->totalFrames == 0)
+	    {
+		return FLASH_PARSE_ERROR;
 	    }
 
 	    status |= FLASH_PARSE_START;
-    }
+	    needHeader = FALSE;
+	}
 
-    ParseTags(&status);
-    return status;
+	ParseTags(&status);
+
+	// If we've reached the end of a compressed movie, we need to shut down
+	// the decompressor and dispose of the decompressed data.
+	if( m_zBuffer && (status & FLASH_PARSE_EOM) )
+	{
+	    inflateEnd(&m_zstream);
+	    m_zInitialized = false;
+
+	    delete m_zBuffer;
+	    m_zBuffer = NULL;
+	}
+
+	return status;
 }
 
 
