@@ -37,7 +37,7 @@
 #endif
 
 #ifdef RCSID
-static char *rcsid = "$Id: sound.cc,v 1.11 1999/01/31 21:14:22 olivier Exp $";
+static char *rcsid = "$Id: sound.cc,v 1.7 2004/12/23 17:13:22 tgc Exp $";
 #endif
 
 #define PRINT 0
@@ -50,6 +50,7 @@ Sound::Sound(long id) : Character(SoundType, id)
 	stereo = 0;
 	soundRate = 0;
 	sampleSize = 1;
+	playbackStarted = 0;
 }
 
 Sound::~Sound()
@@ -82,6 +83,9 @@ Sound::setSoundFlags(long f) {
 		stereo = 1;
 	}
 
+	format = GET_SOUND_FORMAT_CODE(f);
+	
+
 #if PRINT
 	printf("-----\nFlags = %2x\n", f);
 	printf("Rate = %d kHz  ", soundRate);
@@ -107,13 +111,59 @@ Sound::setNbSamples(long n) {
 
 	size = nbSamples * (stereo ? 2 : 1) * sampleSize;
 
-	samples = new char[ size ];
+	return setBuffer(size);
+}
 
-	memset((char *)samples,0, size);
+char *
+Sound::setBuffer(long n) {
+
+	samples = new char[ n ];
+
+	memset((char *)samples,0, n);
 
 	return samples;
 }
 
+char *
+Sound::resetBuffer(long n) {
+
+	char* tmpSamples;
+	long tmpSoundSize;	
+
+	tmpSoundSize = n + soundSize;
+	
+	tmpSamples = samples;
+
+	samples = new char[ tmpSoundSize ];
+
+	memset((char *)samples,0, tmpSoundSize);
+	memcpy(samples, tmpSamples, soundSize);
+	delete tmpSamples;
+
+	soundSize = tmpSoundSize;
+	return samples;
+}
+
+void
+Sound::setSoundSize(long n) {
+	soundSize=n;
+}
+
+void
+Sound::setPlaybackStarted() {
+	playbackStarted=1;
+}
+
+void
+Sound::setSound(SoundList *sl) {
+	sound=sl;
+}
+
+long
+Sound::getFormat() {
+	return format;
+}
+
 long
 Sound::getRate() {
 	return soundRate;
@@ -139,6 +189,21 @@ Sound::getSamples() {
 	return samples;
 }
 
+long
+Sound::getSoundSize() {
+	return soundSize;
+}
+
+SoundList *
+Sound::getSound() {
+	return sound;
+}
+
+long
+Sound::getPlaybackStarted() {
+	return playbackStarted;
+}
+
 //////////// SOUND MIXER
 
 long  SoundMixer::dsp = -1;	// Init of descriptor
@@ -162,7 +227,7 @@ SoundMixer::SoundMixer(char *device)
 		return;
 	}
 
-	dsp = open(device,O_WRONLY);
+	dsp = open(device,O_WRONLY | O_NONBLOCK);
 	if (dsp < 0) {
 		perror("open dsp");
 		return;
@@ -205,7 +270,6 @@ SoundMixer::SoundMixer(char *device)
 		blockSize = 32768;
 	}
 	blockSize *= 2;
-
 	buffer = (char *)malloc(blockSize);
 	if (buffer == 0) {
 		close(dsp);
@@ -234,6 +298,7 @@ SoundMixer::~SoundMixer()
 			free(buffer);
 		}
 	}
+	stopSounds();
 }
 
 void
@@ -245,6 +310,10 @@ SoundMixer::stopSounds()
 	for(sl = list; sl; ) {
 		del = sl;
 		sl = sl->next;
+
+		// If MP3-format, de-init the mad-decoder
+		if (del->currentMp3) uninitMp3Sounds(del);
+
 		delete del;
 	}
 	list = 0;
@@ -258,19 +327,61 @@ SoundMixer::startSound(Sound *sound)
 	SoundList *sl;
 
 	if (sound) {
+
 		// Add sound in list
 		sl = new SoundList;
 		sl->rate = sound->getRate();
 		sl->stereo = (sound->getChannel() == 2);
 		sl->sampleSize = sound->getSampleSize();
-		sl->current = sound->getSamples();
-		sl->remaining = sound->getSampleSize()*sound->getNbSamples()*sound->getChannel();
+		if (sound->getFormat() == 2) {
+			//Mp3 specific data
+			sl->currentMp3 = sound->getSamples();
+			sl->remainingMp3 = sound->getSoundSize();
+			sl->originalMp3 = sl->currentMp3;
+			sl->remaining = 0;
+			
+			// Init the mad-decoder
+			mad_stream_init (&sl->mp3Stream);
+			mad_frame_init (&sl->mp3Frame);
+			mad_synth_init (&sl->mp3Synth);
+			mad_stream_buffer (&sl->mp3Stream, (unsigned char*)sl->currentMp3, sl->remainingMp3);
+
+			fprintf(stderr,"init mp3-decoder, size: %d\n", sound->getSoundSize());
+
+			// make the buffer ready
+			sl->current = (char*) malloc(1200);
+			memset((void*)sl->current, 0, 1200);
+
+			// save the starting point of the buffer
+			sl->original = sl->current;
+
+		} else {
+			sl->current = sound->getSamples();
+			sl->remaining = sound->getSampleSize()*sound->getNbSamples()*sound->getChannel();
+			sl->remainingMp3 = 0;
+		}
 		sl->next = list;
 		list = sl;
+		sound->setPlaybackStarted();
+		sound->setSound(sl);
 	}
 #endif
 }
 
+void
+SoundMixer::uninitMp3Sounds(SoundList *sl)
+{
+#ifndef NOSOUND
+	fprintf(stderr,"dismantle mp3-decoder\n");
+ 	mad_synth_finish (&sl->mp3Synth);
+	mad_frame_finish (&sl->mp3Frame);
+	mad_stream_finish (&sl->mp3Stream);
+	// Seems it sometimes is free before this ???
+	//free(sl->current);
+#endif
+}
+
+
 long
 SoundMixer::playSounds()
 {
@@ -312,7 +423,13 @@ SoundMixer::playSounds()
 		}
 
 		// No more samples for this sound
-		if (sl->remaining == 0) {
+		if ((sl->remaining == 0) && (sl->remainingMp3==0)) {
+
+			// If MP3-format, de-init the mad-decoder
+			if (sl->currentMp3) {
+				uninitMp3Sounds(sl);
+			}
+
 			// Remove sound from list
 			if (prev) {
 				prev->next = sl->next;
@@ -324,6 +441,7 @@ SoundMixer::playSounds()
 				sl = list;
 			}
 		} else {
+			prev = sl;
 			sl = sl->next;
 		}
 	}
@@ -366,6 +484,12 @@ SoundMixer::fillSoundBuffer(SoundList *sl, char *buff, long buffSize)
 
 	skipOut = skipOutInit;
 	skipIn = skipInInit;
+
+	// If mp3-format, and if there is something to decode and
+	// there is nothing in the buffer, decode and fill the buffer
+
+	if ((sl->remainingMp3 > 0) && (sl->remaining < 1)) Mp3Decompress(sl);
+
 	while (buffSize && sl->remaining) {
 		if (skipIn-- == 0) {
 			// Get sampleLeft
@@ -440,7 +564,116 @@ SoundMixer::fillSoundBuffer(SoundList *sl, char *buff, long buffSize)
 
 			skipOut = skipOutInit;
 		}
+
+		// If mp3-format, and if there is something to decode and
+		// there is nothing in the buffer, decode and fill the buffer
+		if ((sl->remainingMp3 >0)&& (sl->remaining < 1)) Mp3Decompress(sl);
 	}
 
 	return totalOut;
 }
+
+
+/* Taken from 'minimad.c'
+ * The following utility routine performs simple rounding, clipping, and
+ * scaling of MAD's high-resolution samples down to 16 bits. It does not
+ * perform any dithering or noise shaping, which would be recommended to
+ * obtain any exceptional audio quality. It is therefore not recommended to
+ * use this routine if high-quality output is desired.
+ */
+int SoundMixer::Mp3Scale(mad_fixed_t sample)
+{
+	/* round */
+	sample += (1L << (MAD_F_FRACBITS - 16));
+
+	/* clip */
+	if (sample >= MAD_F_ONE)
+		sample = MAD_F_ONE - 1;
+	else if (sample < -MAD_F_ONE)
+		sample = -MAD_F_ONE;
+
+	/* quantize */
+	return sample >> (MAD_F_FRACBITS + 1 - 16);
+}
+
+void
+SoundMixer::Mp3Decompress(SoundList *sl)
+{
+	int ret;
+	signed int sample;
+
+	// reset the pointer to the start of the buffer
+	sl->current = sl->original;
+
+	// the actual decoding
+	ret = mad_frame_decode (&sl->mp3Frame, &sl->mp3Stream);
+	
+	if (ret == -1) {
+		switch (sl->mp3Stream.error) {
+			case MAD_ERROR_BUFLEN:
+				// End of buffer - no more to decode
+				sl->remainingMp3=0;
+				return;
+			case MAD_ERROR_BUFPTR:
+			case MAD_ERROR_NOMEM:
+				fprintf(stderr,"MAD: out of memory\n");
+				sl->remainingMp3=0;
+				return;
+			case MAD_ERROR_BADCRC:
+				fprintf(stderr,"MAD: checksum error\n");
+				sl->remainingMp3=0;
+				return;
+			case MAD_ERROR_LOSTSYNC:
+				fprintf(stderr,"MAD: synchronization lost\n");
+				sl->remainingMp3=0;
+				return;
+				case MAD_ERROR_BADLAYER:
+			case MAD_ERROR_BADBITRATE:
+			case MAD_ERROR_BADSAMPLERATE:
+			case MAD_ERROR_BADEMPHASIS:
+			case MAD_ERROR_BADBITALLOC:
+			case MAD_ERROR_BADSCALEFACTOR:
+			case MAD_ERROR_BADFRAMELEN:
+			case MAD_ERROR_BADBIGVALUES:
+			case MAD_ERROR_BADBLOCKTYPE:
+			case MAD_ERROR_BADSCFSI:
+			case MAD_ERROR_BADDATAPTR:
+			case MAD_ERROR_BADPART3LEN:
+			case MAD_ERROR_BADHUFFTABLE:
+			case MAD_ERROR_BADHUFFDATA:
+			case MAD_ERROR_BADSTEREO:
+				fprintf(stderr,"MAD: file contains invalid data\n");
+				sl->remainingMp3=0;
+				return;
+			default:
+				fprintf(stderr,"MAD: unknown error 0x%X.\n",sl->mp3Stream.error);
+				sl->remainingMp3=0;
+				return;
+
+		}
+	}
+	mad_synth_frame (&sl->mp3Synth, &sl->mp3Frame);
+
+	// stereo or not?
+	int sn = sl->stereo ? 2 : 1;
+
+	// Set the size
+	sl->remaining = sl->mp3Synth.pcm.length * sl->sampleSize * sn;
+
+	// Setup the buffer
+	short *dst = (short *) sl->current;
+
+	// transfer the decoded samples into the buffer
+	for(int i = 0; i < sl->mp3Synth.pcm.length; i++)
+	{
+		for (int e = 0; e < sn; e++){
+			sample = Mp3Scale(sl->mp3Synth.pcm.samples[e][i]);
+
+			if ( sample != (short)sample )
+				sample = sample < 0 ? -32768 : 32767;
+			
+			*dst++ = sample;
+		}
+	}
+
+}
