@@ -7,7 +7,7 @@
  *	by Olivier Debon <odebon@club-internet.fr>
  */
 
-static char *rcsid = "$Id: plugin.c,v 1.10 1998/12/06 18:31:53 olivier Exp olivier $";
+static char *rcsid = "$Id: plugin.c,v 1.3 2004/12/22 19:36:20 tgc Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -26,6 +26,7 @@ static char *rcsid = "$Id: plugin.c,v 1.10 1998/12/06 18:31:53 olivier Exp olivi
 #include <X11/IntrinsicP.h>
 #include <X11/Core.h>
 #include <X11/CoreP.h>
+#include <pthread.h>
 
 #define PRINT 0
 #define FLASH_XEVENT_MASK (ExposureMask|ButtonReleaseMask|ButtonPressMask|PointerMotionMask|KeyPressMask|KeyReleaseMask)
@@ -70,6 +71,7 @@ static long FlashExecX11(PluginInstance *, long , XEvent *, struct timeval *);
 static long FlashGraphicInitX11(PluginInstance* This);
 static void FlashCopyX11(PluginInstance* This);
 
+static pthread_mutex_t synchro;
 
 #ifdef C6R5
 // Ugly but it makes it work if it is compiled on
@@ -99,7 +101,7 @@ NPP_GetValue(void *future, NPPVariable variable, void *value)
 			*((char **)value) = "Flash Movie player " FLASH_VERSION_STRING
 					    " compatible with Shockwave Flash 4.0 "
 					    "<P>Shockwave is a trademark of <A HREF=\"http://www.macromedia.com\">Macromedia&reg;</A>"
-					    "<P>Author : <A HREF=mailto:odebon@club-internet.fr>Olivier Debon </A>";
+			  "<P>GPLFLash homepage : <A HREF=\"http://gplflash.sf.net\">gplflash.sf.net</A>";
 			break;
 		default:
 			err = NPERR_GENERIC_ERROR;
@@ -182,8 +184,7 @@ NPP_Destroy(NPP instance, NPSavedData** save)
 	This = (PluginInstance*) instance->pdata;
 
 	if (This != NULL) {
-		LoadCtxt *l,*prev;
-
+		pthread_mutex_lock(&synchro);
 		if (This->timer) {
 			XtRemoveTimeOut(This->timer);
 			This->timer = 0;
@@ -198,26 +199,10 @@ NPP_Destroy(NPP instance, NPSavedData** save)
 			This->fh = 0;
 		}
 
-/*	If this line is included mozilla crashes 50%-75% of the time when leaving a
-	page with flash-plugin, probably because mozilla already has removed the handler.
-	Somebody with actual knowledge about this should try to make a real fix.
-
-
-		XtRemoveRawEventHandler(This->widget, FLASH_XEVENT_MASK,
-				  True, (XtEventHandler) flashEvent, (XtPointer)This);
-*/
-
-		prev = 0;
-		for(l = This->loading; l; prev = l, l = l->next) {
-			free(l->data);
-			free(l->url);
-			free(prev);
-		}
-		free(prev);
-
-		NPN_MemFree(instance->pdata);
-		instance->pdata = NULL;
-	}
+    		NPN_MemFree(instance->pdata); 
+    		instance->pdata = NULL; 
+		pthread_mutex_unlock(&synchro); 
+  	}	 
 
 	return NPERR_NO_ERROR;
 }
@@ -272,6 +257,7 @@ flashEvent(Widget w, XtPointer client_data, XEvent *event, Boolean *dispatch)
 static void
 flashWakeUp(XtPointer client_data, XtIntervalId *id)
 {
+	pthread_mutex_lock(&synchro);
 	PluginInstance* This;
 	long cmd;
 	long wakeUp;
@@ -295,6 +281,7 @@ flashWakeUp(XtPointer client_data, XtIntervalId *id)
 			This->timer = 0;
 		}
 	}
+	pthread_mutex_unlock(&synchro);
 }
 
 NPError 
@@ -446,7 +433,8 @@ getSwf(char *url, int level, void *client_data)
 	if (This == 0) return;
 
 	l = (LoadCtxt *)malloc(sizeof(LoadCtxt));
-	l->next = This->loading;
+	//????? Cause memory area to be deallocated multiple times. needed?: l->next = This->loading; 
+	l->next = 0; 
 	This->loading = l;
 	l->level = level;
 	l->data = 0;
@@ -495,20 +483,23 @@ NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
 
 	if (This == 0) return NPERR_INVALID_INSTANCE_ERROR;
 
-	if (This->fh == 0) {
-		return NPERR_INVALID_INSTANCE_ERROR;
-	}
-
+	//if (This->fh == 0) {
+	//	return NPERR_INVALID_INSTANCE_ERROR;
+	//}
+	if ((NPP)This->loading == (NPP)instance ) return NPERR_NO_ERROR; // Nothing to deallocate 
 	for(l = This->loading; l; l = l->next) {
 		if (l->url && strstr(stream->url, l->url)) {
-			free(l->data);
+			// Troubles here! Sometimes seems to be already freed (firefox). 
+			// Someone knows what's happening here? 
+			//free(l->data);	// Leak! 
 			l->data = 0;
-			free(l->url);
+			//free(l->url);		// Leak! 
 			l->url = 0;
 			break;
 		}
 	}
-
+	// Forbidden: cannot use any X11 commands since context may be invalid 
+#if 0 
 	if (!This->gInitDone && This->dpy) {
 		FlashGraphicInitX11(This);
 		XtAddEventHandler(This->widget, FLASH_XEVENT_MASK,
@@ -517,7 +508,7 @@ NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
 
 		flashWakeUp((XtPointer)This, 0);
 	}
-
+#endif
 	return NPERR_NO_ERROR;
 }
 
